

#ifndef _SERIAL_H
#define _SERIAL_H

#define USE_FFUART
#undef  USE_BTUART
#undef  USE_STUART

#define UART_BAUDRATE			(115200)
//#define LCR_VALUE			(0x03)		// No Parity, 8 bit data, 1 stop bit

// Change between BaudRate and DLV (Value of Divisor Latch Registers:DLL,DLH)
// PXA255 Processor Developer's Manual
//   10.4.2.3 Divisor Latch Registers (DLL and DLH) p 10-7
#define BaudToDLV(_baud)				(14745600 / (16 * (_baud)))
#define DLVToBaud(_dlv)					(14745600 / (16 * (_dlv)))


#define UART_RBR					(0x00)
#define UART_THR					(0x00)
#define UART_IER					(0x04)
#define UART_IIR					(0x08)
#define UART_FCR					(0x08)
#define UART_LCR					(0x0C)
#define UART_MCR					(0x10)
#define UART_LSR					(0x14)
#define UART_MSR					(0x18)
#define UART_SPR					(0x1C)
#define UART_ISR					(0x20)
#define UART_DLL					(0x00)
#define UART_DLH					(0x04)

#define UART_IER_DMAE				(0x80)
#define UART_IER_UUE				(0x40)
#define UART_IER_NRZE				(0x20)
#define UART_IER_RTOIE				(0x10)
#define UART_IER_MIE				(0x08)
#define UART_IER_RLSE				(0x04)
#define UART_IER_TIE				(0x02)
#define UART_IER_RAVIE				(0x01)

#define UART_IIR_FIFOES_MASK		(0x0C)
#define UART_IIR_FIFOES_NOT_FIFO	(0x00)
#define UART_IIR_FIFOES_FIFO		(0x0C)
#define UART_IIR_TOD				(0x08)
#define UART_IIR_IID_MASK			(0x06)
#define UART_IIR_IID_MODEM_STATUS	(0x00)
#define UART_IIR_IID_TRD			(0x02)
#define UART_IIR_IID_RDA			(0x04)
#define UART_IIR_IID_RERR			(0x06)
#define UART_IIR_IP					(0x01)

#define UART_LSR_FIFOE				(0x80)
#define UART_LSR_TEMT				(0x40)
#define UART_LSR_TDRQ				(0x20)
#define UART_LSR_BI					(0x10)
#define UART_LSR_FE					(0x08)
#define UART_LSR_PE					(0x04)
#define UART_LSR_OE					(0x02)
#define UART_LSR_DR					(0x01)

#ifndef __ASSEMBLY__
extern bool uart_init(void);
extern int uart_putc(int ch);			// put char : \n -> \r\n
extern int uart_getc(void);				// get char : \r -> \n
extern int uart_puts(const char *s);

extern int uart_putb(int ch);			// put byte
extern int uart_getb(void);				// get byte
extern int uart_write(const void *buff, int count);
#endif

#endif

