

#include <config.h>
#include <hardware.h>
#include <board.h>

.global setup_memory

#define SDRAM_BASE			0xA0000000

#define CLKCFG_FAST_BUS		0x08
#define CLKCFG_B			0x08
#define CLKCFG_HALF_TURBO	0x04
#define CLKCFG_HT			0x04
#define CLKCFG_FCS			0x02
#define CLKCFG_F			0x02
#define CLKCFG_TURBO		0x01


#define MEMCTL_BASE			0x48000000
#define OMDCNFG				0x00			/* SDRAM Configuration Register 0 */
#define OMDREFR				0x04			/* SDRAM Refresh Control Register */
#define OMSC0				0x08			/* Static Memory Control Register 0 */
#define OMSC1				0x0C			/* Static Memory Control Register 1 */
#define OMSC2				0x10			/* Static Memory Control Register 2 */
#define OMECR				0x14			/* Expansion Memory (PCMCIA/Compact Flash) Bus Configuration */
#define OSXLCR				0x18			/* LCR value to be written to SDRAM-Timing Synchronous Flash */
#define OSXCNFG				0x1C			/* Synchronous Static Memory Control Register */
#define OFLYCNFG			0x20			/* Fly-by DMA DVAL<1:0> polarities */
#define OSXMRS				0x24			/* MRS value to be written to Synchronous Flash or SMROM */
#define OMCMEM0				0x28			/* Card interface Common Memory Space Socket 0 Timing */
#define OMCMEM1				0x2C			/* Card interface Common Memory Space Socket 1 Timing */
#define OMCATT0				0x30			/* Card interface Attribute Space Socket 0 Timing Configuration */
#define OMCATT1				0x34			/* Card interface Attribute Space Socket 1 Timing Configuration */
#define OMCIO0				0x38			/* Card interface I/O Space Socket 0 Timing Configuration */
#define OMCIO1				0x3C			/* Card interface I/O Space Socket 1 Timing Configuration */
#define OMDMRS				0x40			/* MRS value to be written to SDRAM */
#define OBOOT_DEF			0x44			/* Read-Only Boot-Time Register. Contains BOOT_SEL and PKG_SEL */
#define OSA1110				0x64			/* SA-1110 Compatibility Mode for Static Memory Register */


#ifdef CONFIG_PXA25x

/***** Setup Static & Dynamic Memory *****/
.func setup_memory
setup_memory :
	ldr		r1, =CCCR
	ldr		r0, =_CCCR
	str		r0, [r1]					@ set CCCR
	mov		r0, #(CLKCFG_TURBO | CLKCFG_FCS)
	mcr		p14, 0, r0, c6, c0, 0		@ enter the frequency change sequence

	ldr		r1, =OSCR
	ldr		r0, [r1]
	add		r2, r0, #0x300
 1:	ldr		r0, [r1]
	cmp		r0, r2
	blt		1b

	mov		r3, #MEMCTL_BASE
	ldr		r2, =_MSC0
	bic		r2, r2, #0x08
	ldr		r0, [r3, #OMSC0]
	and		r1, r0, #0x08
	orr		r0, r1, r2
	str		r0, [r3, #OMSC0]
	ldr		r0, [r3, #OMSC0]
	ldr		r0, =_MSC1
	str		r0, [r3, #OMSC1]
	ldr		r0, [r3, #OMSC1]
	ldr		r0, =_MSC2
	str		r0, [r3, #OMSC2]
	ldr		r0, [r3, #OMSC2]

	ldr		r0, =_MECR
	str		r0, [r3, #OMECR]
	ldr		r0, [r3, #OMECR]
	ldr		r0, =_MCMEM0
	str		r0, [r3, #OMCMEM0]
	ldr		r0, [r3, #OMCMEM0]
	ldr		r0, =_MCMEM1
	str		r0, [r3, #OMCMEM1]
	ldr		r0, [r3, #OMCMEM1]
	ldr		r0, =_MCATT0
	str		r0, [r3, #OMCATT0]
	ldr		r0, [r3, #OMCATT0]
	ldr		r0, =_MCATT1
	str		r0, [r3, #OMCATT1]
	ldr		r0, [r3, #OMCATT1]
	ldr		r0, =_MCIO0
	str		r0, [r3, #OMCIO0]
	ldr		r0, [r3, #OMCIO0]
	ldr		r0, =_MCIO1
	str		r0, [r3, #OMCIO1]
	ldr		r0, [r3, #OMCIO1]

	ldr		r1, =_MDREFR
	ldr		r0, =(MDREFR_K0RUN | MDREFR_E0PIN | MDREFR_K0DB2)
	and		r1, r1, r0
	ldr		r2, [r3, #OMDREFR]
	ldr		r0, =(0xFFF | MDREFR_APD | MDREFR_SLFRSH)
	and		r2, r2, r0
	orr		r0, r1, r2
	str		r1, [r3, #OMDREFR]
	ldr		r1, [r3, #OMDREFR]

	ldr		r0, =_SXCNFG
	str		r0, [r3, #OSXCNFG]
	ldr		r0, [r3, #OSXCNFG]

	ldr		r1, [r3, #OMDREFR]
	ldr		r0, =_MDREFR
	and		r0, r0, #(MDREFR_K1RUN | MDREFR_K2RUN | MDREFR_K1DB2 | MDREFR_K2DB2)
	orr		r1, r1, r0
	str		r1, [r3, #OMDREFR]

	ldr		r1, [r3, #OMDREFR]
	bic		r1, r1, #MDREFR_SLFRSH
	str		r1, [r3, #OMDREFR]
	ldr		r1, [r3, #OMDREFR]

	ldr		r0, =_MDREFR
	and		r0, r0, #MDREFR_E1PIN
	orr		r1, r1, r0
	str		r1, [r3, #OMDREFR]
	ldr		r1, [r3, #OMDREFR]

	ldr		r0, =_MDCNFG
	bic		r0, r0, #(MDCNFG_DE0 | MDCNFG_DE1)
	bic		r0, r0, #(MDCNFG_DE2 | MDCNFG_DE3)
	str		r0, [r3, #OMDCNFG]
	ldr		r0, [r3, #OMDCNFG]

	ldr		r1, =OSCR
	ldr		r0, [r1]
	add		r2, r0, #0x300
 1:	ldr		r0, [r1]
	cmp		r0, r2
	blt		1b

	mrc		p15, 0, r0, c1, c0, 0
	bic		r0, r0, #0x04
	mcr		p15, 0, r0, c1, c0, 0

	ldr		r1, =RCSR
	ldr		r0, [r1]
	ands	r0, r0, #RCSR_HWR
	beq		2f

	ldr		r0, =SDRAM_BASE
  .rept 8
	str		r0, [r0]
  .endr
 2:

	ldr		r0, =_MDCNFG
	str		r0, [r3, #OMDCNFG]

	ldr		r0, =_MDMRS
	str		r0, [r3, #OMDMRS]

	ldr		r0, =_MDREFR
	str		r0, [r3, #OMDREFR]

	mov		pc, lr				@ return
.endfunc

#endif


#ifdef CONFIG_PXA27x

.func setup_memory
setup_memory :
	ldr		r1, =CCCR
	ldr		r0, =_CCCR
	str		r0, [r1]					@ set CCCR
	mov		r0, #(CLKCFG_TURBO | CLKCFG_FCS)
	mcr		p14, 0, r0, c6, c0, 0		@ enter the frequency change sequence

	ldr		r1, =OSCR
	ldr		r0, [r1]
	add		r2, r0, #0x300
 1:	ldr		r0, [r1]
	cmp		r0, r2
	blt		1b

	mov		r3, #MEMCTL_BASE
	ldr		r2, =_MSC0
	bic		r2, r2, #0x08
	ldr		r0, [r3, #OMSC0]
	and		r1, r0, #0x08
	orr		r0, r1, r2
	str		r0, [r3, #OMSC0]
	ldr		r0, [r3, #OMSC0]
	ldr		r0, =_MSC1
	str		r0, [r3, #OMSC1]
	ldr		r0, [r3, #OMSC1]
	ldr		r0, =_MSC2
	str		r0, [r3, #OMSC2]
	ldr		r0, [r3, #OMSC2]
	ldr		r0, =_SA1110
	str		r0, [r3, #OSA1110]

	ldr		r0, =_MECR
	str		r0, [r3, #OMECR]
	ldr		r0, [r3, #OMECR]
	ldr		r0, =_MCMEM0
	str		r0, [r3, #OMCMEM0]
	ldr		r0, [r3, #OMCMEM0]
	ldr		r0, =_MCMEM1
	str		r0, [r3, #OMCMEM1]
	ldr		r0, [r3, #OMCMEM1]
	ldr		r0, =_MCATT0
	str		r0, [r3, #OMCATT0]
	ldr		r0, [r3, #OMCATT0]
	ldr		r0, =_MCATT1
	str		r0, [r3, #OMCATT1]
	ldr		r0, [r3, #OMCATT1]
	ldr		r0, =_MCIO0
	str		r0, [r3, #OMCIO0]
	ldr		r0, [r3, #OMCIO0]
	ldr		r0, =_MCIO1
	str		r0, [r3, #OMCIO1]
	ldr		r0, [r3, #OMCIO1]

	ldr		r0, =_FLYCNFG
	str		r0, [r3, #OFLYCNFG]
	ldr		r0, [r3, #OFLYCNFG]

	ldr		r0, =_MDCNFG
	bic		r0, r0, #(MDCNFG_DE0 | MDCNFG_DE1)		@ disable sdram bank 0, 1
	bic		r0, r0, #(MDCNFG_DE2 | MDCNFG_DE3)		@ disable sdram bank 0, 1
	str		r0, [r3, #OMDCNFG]
	ldr		r0, [r3, #OMDCNFG]

	ldr		r0, [r3, #OMDREFR]
	ldr		r2, =0xFFF
	and		r1, r0, r2
	ldr		r0, =_MDREFR
	ldr		r2, =(MDREFR_K0RUN | MDREFR_K0DB2 | MDREFR_K0DB4 | MDREFR_K0FREE | MDREFR_K1FREE | MDREFR_K2FREE)
	and		r0, r0, r2
	orr		r1, r1, r0
	bic		r1, r1, #MDREFR_APD
	orr		r1, r1, #MDREFR_SLFRSH
	str		r1, [r3, #OMDREFR]
	ldr		r1, [r3, #OMDREFR]

	ldr		r0, =_SXCNFG
	str		r0, [r3, #OSXCNFG]
	ldr		r0, [r3, #OSXCNFG]

	ldr		r1, [r3, #OMDREFR]
	ldr		r0, =_MDREFR
	and		r0, r0, #(MDREFR_K1RUN | MDREFR_K2RUN | MDREFR_K1DB2 | MDREFR_K2DB2)
	orr		r1, r1, r0
	str		r1, [r3, #OMDREFR]

	ldr		r1, [r3, #OMDREFR]
	bic		r1, r1, #MDREFR_SLFRSH
	str		r1, [r3, #OMDREFR]
	ldr		r1, [r3, #OMDREFR]

	ldr		r0, =_MDREFR
	and		r0, r0, #MDREFR_E1PIN
	orr		r1, r1, r0
	str		r1, [r3, #OMDREFR]
	ldr		r1, [r3, #OMDREFR]

	ldr		r0, =_MDCNFG
	bic		r0, r0, #(MDCNFG_DE0 | MDCNFG_DE1)		@ disable sdram bank 0, 1
	bic		r0, r0, #(MDCNFG_DE2 | MDCNFG_DE3)		@ disable sdram bank 0, 1
	str		r0, [r3, #OMDCNFG]

	ldr		r1, =OSCR
	ldr		r0, [r1]
	add		r2, r0, #0x300
 1:	ldr		r0, [r1]
	cmp		r0, r2
	blt		1b

	mrc		p15, 0, r0, c1, c0, 0
	bic		r0, r0, #0x04
	mcr		p15, 0, r0, c1, c0, 0

	ldr		r1, =RCSR
	ldr		r0, [r1]
	tst		r0, #RCSR_HWR
	beq		2f				@ if not hardware reset, do nothing.

	ldr		r0, =SDRAM_BASE
  .rept 8
	str		r0, [r0]
  .endr
 2:

	ldr		r0, =_MDCNFG
	str		r0, [r3, #OMDCNFG]

	ldr		r0, =_MDMRS
	str		r0, [r3, #OMDMRS]

	ldr		r0, =_MDREFR
	str		r0, [r3, #OMDREFR]

	mov		pc, lr				@ return
.endfunc

#endif
